/* -*- Mode: C -*- */

#ifdef __cplusplus
extern "C" {
#endif

#ifndef ICALSTRINGSET_H
#define ICALSTRINGSET_H

#include <libical/ical.h>
#include <libical/icalset.h>
#include <libical/icalcluster.h>
#include <libical/icalgauge.h>

typedef struct icalstringset_impl icalstringset;

icalset* icalstringset_new(const char* icalendar_str);

icalset* icalstringset_init(icalset *set, const char *icalendar_str, void* options);

void icalstringset_free(icalset* cluster);

const char* icalstringset_icalendarstrring(icalset* cluster);

/* Mark the cluster as changed, so it will be written to disk when it
   is freed. Commit writes to disk immediately. */
void icalstringset_mark(icalset* set);
icalerrorenum icalstringset_commit(icalset* set); 

icalerrorenum icalstringset_add_component(icalset* set,
					icalcomponent* child);

icalerrorenum icalstringset_remove_component(icalset* set,
					   icalcomponent* child);

int icalstringset_count_components(icalset* set,
				 icalcomponent_kind kind);

/**
 * Restrict the component returned by icalstringset_first, _next to those
 * that pass the gauge. _clear removes the gauge 
 */
icalerrorenum icalstringset_select(icalset* set, icalgauge* gauge);

/** clear the gauge **/
void icalstringset_clear(icalset* set);

/** Get and search for a component by uid **/
icalcomponent* icalstringset_fetch(icalset* set, const char* uid);
int icalstringset_has_uid(icalset* set, const char* uid);
icalcomponent* icalstringset_fetch_match(icalset* set, icalcomponent *c);


/**
 *  Modify components according to the MODIFY method of CAP. Works on the
 *  currently selected components. 
 */
icalerrorenum icalstringset_modify(icalset* set, 
				 icalcomponent *oldcomp,
			       icalcomponent *newcomp);

/* Iterate through components. If a gauge has been defined, these
   will skip over components that do not pass the gauge */

icalcomponent* icalstringset_get_current_component (icalset* cluster);
icalcomponent* icalstringset_get_first_component(icalset* cluster);
icalcomponent* icalstringset_get_next_component(icalset* cluster);

/* External iterator for thread safety */
icalsetiter icalstringset_begin_component(icalset* set, icalcomponent_kind kind, icalgauge* gauge);
icalcomponent * icalstringsetiter_to_next(icalset* set, icalsetiter *iter);

/** 
 * @brief options for opening an icalfileset.
 *
 * These options should be passed to the icalset_new() function
 */

typedef struct icalstringset_options {
  icalcluster  *cluster;	/**< use this cluster to initialize data */
} icalstringset_options;

extern icalstringset_options icalstringset_options_default;

#endif /* !ICALSTRINGSET_H */

#ifdef __cplusplus
}
#endif


